# SteelSections.tcl  -*- TCL -*-
#         $Id: SteelSections.tcl,v 1.4 2003/03/27 18:38:36 ramsan Exp $        


namespace eval SteelSections {
    variable dblclick
    variable tree
    variable imglabel
    variable text
    variable DataTypes
    variable Data
    variable Units
    variable Description
    variable Id
    variable Images
    variable material
    variable path
}

proc SteelSections::ReadData { file } {
    variable DataTypes
    variable Data
    variable Units
    variable Description
    variable Id

    catch { unset Description Id Units Data DataTypes }

    set f [open $file r]
    set line 1
    while { ![eof $f] } {
	gets $f aa
	if { [regexp {^[ ]*$} $aa] } { continue }
	if { [regexp {^[ ]*#} $aa] } { continue }
	if { [string match Description* $aa] } {
	    set Description([lindex $aa 1]) [lrange $aa 2 end]
	    lappend DataTypes [lindex $aa 1]
	} elseif { [string match Units* $aa] } {
	    set Units([lindex $aa 1]) [lrange $aa 2 end]
	} elseif { [string match Id* $aa] } {
	    if { ![info exists Description([lindex $aa 1])] } {
		tk_messageBox -message \
		        "type '[lindex $aa 1]' has no Description when reading Id. Line=$line"
		exit
	    }
	    set pos ""
	    foreach i [lrange $aa 2 end] {
		lappend pos [lsearch $Description([lindex $aa 1]) $i]
	    }
	    set Id([lindex $aa 1]) $pos
	} else {
	    set type [lindex $aa 0]
	    if { ![info exists Description($type)] } {
		tk_messageBox -message "type '$i' has no Description. Line=$line"
		exit
	    }
	    if { ![info exists Id($type)] } {
		tk_messageBox -message "type '$i' has no Id. Line=$line"
		exit
	    }
	    set name ""
	    foreach i $Id($type) {
		lappend name [lindex $aa $i]
	    }
	    set Data($name) $aa
	}
	incr line
    }
    close $f
}

proc SteelSections::SortData { idp data1 data2 } {

    set c 0
    foreach i $idp {
	if { [lindex $data1 $c] < [lindex $data2 $c] } { return -1 }
	if { [lindex $data1 $c] > [lindex $data2 $c] } { return 1 }
	incr c
    }
    return 0
}

proc SteelSections::ConvertValuesToSI { val unit } {

    switch $unit {
	cm { return [expr $val*1e-2] }
	mm { return [expr $val*1e-3] }
	cm2 { return [expr $val*1e-4] }
	cm4 { return [expr $val*1e-8] }
	kp/m { return [expr $val*9.8] }
	default {
	    error "Unknown unit '$unit' "
	    #tk_messageBox -message "Unknown unit '$unit' "
	    exit
	}
    }
}

proc SteelSections::ConvertValuesFromSI { val unit } {

    switch $unit {
	cm { return [expr $val*1e2] }
	mm { return [expr $val*1e3] }
	cm2 { return [expr $val*1e4] }
	cm4 { return [expr $val*1e8] }
	kp/m { return [expr $val/9.8] }
	default {
	    error "Unknown unit '$unit' "
	    #tk_messageBox -message "Unknown unit '$unit' "
	    exit
	}
    }
}

proc SteelSections::ConvertValuesWithUnits { val unitfrom unitto } {
    return [ConvertValuesFromSI [ConvertValuesToSI $val $unitfrom] $unitto]
}


proc SteelSections::ChangeColumn { datatype oldname newname { expr "" } } {
    variable Units
    variable Description
    variable Data
    
    set ipos [lsearch $Description($datatype) $oldname]
    if { $ipos == -1 } { return }

    set Description($datatype) [lreplace $Description($datatype) $ipos $ipos $newname]
    if { $expr == "" } { return }

    foreach j [array names Data "$datatype *"] {
	foreach "{} var" [regexp -inline -all {[$]([a-zA-Z_]+)} $expr] {
	    if { $var == $oldname } {
		set jpos [lsearch $Description($datatype) $newname]
	    } else {
		set jpos [lsearch $Description($datatype) $var]
	    }
	    if { $jpos == -1 } {
		error "error ChangeColumn var=$var Description($datatype)=$Description($datatype)"
	    }
	    set $var [ConvertValuesWithUnits [lindex $Data($j) $jpos] \
		    [lindex $Units($datatype) $jpos] [lindex $Units($datatype) $ipos]]
	}
	set Data($j) [lreplace $Data($j) $ipos $ipos [expr $expr]]
    }
}


proc SteelSections::WriteTxtData { file } {
    variable DataTypes
    variable Data
    variable Units
    variable Description
    variable Id

    foreach i $DataTypes {
	switch $i {
	    UPN {
		ChangeColumn UPN c Yg
	    }
	    L {
		ChangeColumn L c Yg {$b-$c}
	    }
	    LD {
		ChangeColumn LD cy Yg {$b-$cy}
		ChangeColumn LD cz Zg {$h-$cz}
	    }
	    T {
		ChangeColumn T z Zg {$h-$z}
	    }
	}
    }

    set f [open $file w]
    foreach i $DataTypes {
	puts $f "Description $i $Description($i)"
	puts $f "Units $i $Units($i)"
	puts -nonewline $f "Id $i "
	foreach j $Id($i) {
	    puts -nonewline $f "[lindex $Description($i) $j] "
	}
	puts $f ""
    }
    foreach i $DataTypes {
	set idp ""
	foreach j $Id($i) {
	    lappend idp [lindex $Description($i) $j]
	}
	foreach j [lsort -command [list SteelSections::SortData $idp] [array names Data "$i *"]] {
	    puts $f $Data($j)
	}
    }
    close $f
}


proc SteelSections::Create { frame } {
    variable tree
    variable imglabel
    variable text

    set pw    [PanedWindow $frame.pw -side top]

    set pane  [$pw add -weight 1]
    set title [TitleFrame $pane.lf -text "steel sections"]
    set sw    [ScrolledWindow [$title getframe].sw \
		  -relief sunken -borderwidth 2]
    set tree  [Tree $sw.tree \
	    -relief flat -borderwidth 0 -width 15 -highlightthickness 0\
	    -redraw 0 -dropenabled 1 -dragenabled 1 \
	    -dragevent 3 \
	    -droptypes { \
	    TREE_NODE    {copy {} move {} link {}} \
	    LISTBOX_ITEM {copy {} move {} link {}} \
	} \
	-opencmd   "SteelSections::ModTree 1" \
	-closecmd  "SteelSections::ModTree 0"]
    $sw setwidget $tree

    if {[string equal "unix" $::tcl_platform(platform)]} {
	bind [winfo toplevel $sw.tree] <4> "$sw.tree yview scroll -5 units"
	bind [winfo toplevel $sw.tree] <5> "$sw.tree yview scroll  5 units"
    }

    set title2 [TitleFrame $pane.lf2 -text "material"]
    tk_optionMenu [$title2 getframe].om SteelSections::material A37 A42 A52
    set SteelSections::material A42
    
    pack $sw    -side top  -expand yes -fill both
    pack [$title2 getframe].om -side top  -expand yes -fill x
    pack $title2 -fill x
    pack $title -fill both -expand yes
    
    set pane [$pw add -weight 2]
    set lf   [TitleFrame $pane.lf -text "visual description"]
    set imglabel [Label [$lf getframe].l -bd 2 -relief raised]
    set lf2 [TitleFrame $pane.lf2 -text "properties"]

    set sw    [ScrolledWindow [$lf2 getframe].sw]
    set text [text $sw.t -height 3 -width 10 -highlightthickness 0 -state disabled -wrap none]
    set textfam [font actual [$text cget -font] -family]
    set textsize [font actual [$text cget -font] -size]
    
    $text tag conf superindex -offset [expr [font metrics [$text cget -font] -linespace]/3] -font \
	    [list $textfam [expr $textsize-2]]
    $text tag conf bold -font [list $textfam $textsize bold]

    $sw setwidget $text
    pack $sw -fill both -expand yes
    bind $text <1> "focus %W"

    pack $lf -fill x -expand 0
    pack $lf2 -fill both -expand yes
    pack $imglabel
    pack $text -fill both -expand 1

    pack $pw -fill both -expand yes

    $tree bindText  <ButtonPress-1>        "SteelSections::Select 1"
    $tree bindText  <Double-ButtonPress-1> "SteelSections::Select 2"
    $tree bindImage  <ButtonPress-1>        "SteelSections::Select 1"
    $tree bindImage  <Double-ButtonPress-1> "SteelSections::Select 2"
    # dirty trick
    foreach i [bind $tree.c] {
	bind $tree.c $i "+ [list after idle [list SteelSections::Select 0 {}]]"
    }
#      set bbox [ButtonBox $frame.bb -spacing 3 -padx 12 -pady 0 -homogeneous 1]
#      $bbox add -text Apply -underline 0
#      $bbox add -text Close -command "exit" -underline 0
#      $bbox setfocus 0
#      pack $bbox -pady 3

#      bind [winfo toplevel $frame] <Alt-a> "$bbox invoke 0"
#      bind [winfo toplevel $frame] <Alt-c> "$bbox invoke 1"

   SteelSections::Init
}

proc SteelSections::InsertPropertyInText { text name val units } {

    if { $val == "-" } { return }
    $text ins end $name bold
    $text ins end "=$val"
    for { set j 0 } { $j < [string length $units] } { incr j } {
	set char [string index $units $j]
	if { [string is digit $char] } {
	    $text ins end $char superindex
	} else {
	    $text ins end $char
	}
    }
    $text ins end " "
}

proc SteelSections::Select { num node } {
    variable dblclick
    variable Images
    variable tree
    variable imglabel
    variable text
    variable Data
    variable Description
    variable Units

    if { $num == 0 } {
	#if { ![winfo exists $node] } { return }
	set node [$tree selection get]
	if { [llength $node] != 1 } { return }
    } elseif { ![$tree exists $node] } {
	TryToCreateNode $node
    }
    set dblclick 1
    if { $num == 1 || $num == 0 } {
       if { $num == 1 && [lsearch [$tree selection get] $node] != -1 } {
	    unset dblclick
	   # it can be used to let the user modify on value in the tree
	   #after 500 "SteelSections::Edit $node"
	   return
	}
	$tree selection set $node
	$tree see $node
	set parent [$tree parent $node]
	while { $parent != "root" } {
	    if { [$tree itemcget $parent -open] == 0 } {
		$tree itemconfigure $parent -open 1
	    }
	    set parent [$tree parent $parent]
	}
	#update
	set name [split $node -]
	set type [lindex $name 0]

	CreateImages $type

	$imglabel configure -image $Images($type) \
		-width [image width $Images($type)]
	$text conf -state normal
	$text del 1.0 end

	if { [info exists Data($name)] } {
	    set main [list h b A \n Iy Iz It]
	    foreach i $main {
		if { $i == "\n" } {
		    $text ins end "\n"
		} else {
		    set pos [lsearch $Description($type) $i]
		    if { $pos != -1 } {
		        InsertPropertyInText $text $i [lindex $Data($name) $pos] \
		                [lindex $Units($type) $pos]
		    }
		}
	    }
	    $text ins end "\n\n\n"
	    set icount 0
	    foreach i $Description($type) {
		if { [string match char* $i] || [string match int* $i] } { continue }
		if { [lsearch $main $i] == -1 } {
		    set pos [lsearch $Description($type) $i]
		    InsertPropertyInText $text $i [lindex $Data($name) $pos] \
		            [lindex $Units($type) $pos]
		    incr icount
		    if { $icount == 3 } { 
		        $text ins end "\n"
		        set icount 0
		    }
		}
	    }
	}
	$text conf -state disabled
    } else {
	if { [$tree itemcget $node -open] == 0 } {
	    $tree itemconfigure $node -open 1
	    set idx 1
	} else {
	    $tree itemconfigure $node -open 0
	    set idx 0
	}
	ModTree $idx $node
    }
}

proc SteelSections::Edit { node } {
    variable dblclick
    variable tree

    if { [info exists dblclick] } {
	return
    }
    if { [lsearch [$tree selection get] $node] != -1 } {
	set res [$tree edit $node [$tree itemcget $node -text]]
	if { $res != "" } {
	    $tree itemconfigure $node -text $res
	    if { [$list exists $node] } {
		$list itemconfigure $node -text $res
	    }
	    $tree selection set $node
	}
	return
    }
}

proc SteelSections::SortIndices { idx1 idx2 } {

    if { [lindex $idx1 1] < [lindex $idx2 1] } { return -1 }
    if { [lindex $idx1 1] > [lindex $idx2 1] } { return 1 }
    if { [lindex $idx1 2] < [lindex $idx2 2] } { return -1 }
    if { [lindex $idx1 2] > [lindex $idx2 2] } { return 1 }
    return 0
}

proc SteelSections::CreateImages { datatype } {
    variable Images
    variable path

    if { [info exists Images($datatype)] } { return }

    set imgname profile_$datatype.gif
    if { [string match profile_HE* $imgname] } {
	set imgname profile_HE.gif
    }
    if { [catch {
	set Images($datatype) [image create photo -file [file join $path images $imgname]]
    }] } {
	if { ![info exists Images(profile_blank.gif)] } {
	    set Images(blank) [image create photo -file [file join $path images \
		                                             profile_blank.gif]]
	}
	set Images($datatype) $Images(blank)
    }
    set imgname icon_$datatype.gif

    if { [string match icon_*F.gif $imgname] } {
	regexp {icon_(.*)F} $imgname {} img
	set imgname icon_$img.gif
    }
    if { $imgname == "icon_U.gif" } {
	set imgname icon_UPN.gif
    }
    if { [string match icon_H* $imgname] } {
	set imgname icon_H.gif
    }
    if { [string match icon_IP* $imgname] } {
	set imgname icon_IP.gif
    }
    if { [string match icon_L* $imgname] } {
	set imgname icon_L.gif
    }
    if { [catch {
	set Images(icon,$datatype) [image create photo -file [file join $path images $imgname]]
    }] } {
	if { ![info exists Images(icon_blank.gif)] } {
	    set Images(icon,blank) [image create photo -file [file join $path images \
		                                                  icon_blank.gif]]
	}
	set Images(icon,$datatype) $Images(icon,blank)
    }
}

proc SteelSections::Init { args } {
    variable DataTypes
    variable Data
    variable Units
    variable Description
    variable Id
    variable Images
    variable tree
    variable path

    if { ![info exists DataTypes] } {
	ReadData [file join $path scripts SteelSections.txt]
    }

#     foreach i $DataTypes {
#         SteelSections::CreateImages $i
#     }

    foreach i $DataTypes {
	$tree insert end root $i -text $i -open 0 \
		-image [Bitmap::get folder] -drawcross allways -data $i
#         foreach j [lsort -command SortIndices [array names Data "$i *"]] {
#             $tree insert end $i [join $j -] -text [lrange $j 1 end] -open 0 \
#                     -image $Images(icon,$i)
#         }
    }
    Select 1 [lindex $DataTypes 0]
    $tree configure -redraw 1
}

proc SteelSections::TryToCreateNode { node } {
    variable DataTypes
    variable Data
    variable Images
    variable tree

    foreach i $DataTypes {
	foreach j [lsort -command SortIndices [array names Data "$i *"]] {
	    if { $node == [join $j -] } {
		CreateImages $i
		$tree insert end $i [join $j -] -text [lrange $j 1 end] -open 0 \
		    -image $Images(icon,$i)
		return 1
	    }
	}
    }
    return 0
}

proc SteelSections::ModTree { idx node } {
    variable tree
    variable Data
    variable Images

    if { $idx && [$tree itemcget $node -drawcross] == "allways" } {
	set data [$tree itemcget $node -data]
	CreateImages $data
	foreach j [lsort -command SortIndices [array names Data "$data *"]] {
	    $tree insert end $data [join $j -] -text [lrange $j 1 end] -open 0 \
		-image $Images(icon,$data)
	}
	$tree itemconfigure $node -drawcross auto
    }
    if { [llength [$tree nodes $node]] } {
	if { $idx } {
	    $tree itemconfigure $node -image [Bitmap::get openfold]
	} else {
	    $tree itemconfigure $node -image [Bitmap::get folder]
	}
    }
}

proc SteelSections::GiveSelectedValues {} {
    variable tree
    variable material

    set node [$tree selection get]
    if { [llength $node] != 1 } { return "" }

    if { [$tree itemcget $node -drawcross] == "allways" } {
	# to update non filled nodes
	ModTree 1 $node
	ModTree 0 $node
    }

    if { [$tree nodes $node] != "" } { return "" }
    return [list $node $material]
}

proc SteelSections::ComunicateWithGiD { op args } {
    global ProblemTypePriv

    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]

	    set f [frame $PARENT.f]
	    set SteelSections::path $ProblemTypePriv(problemtypedir)
	    SteelSections::Create $f
	    # not sure if can be commented
	    #update
	    upvar \#0 $GDN GidData
	    if { $GidData($STRUCT,VALUE,2) != "-" } {
		SteelSections::Select 1 $GidData($STRUCT,VALUE,2)
	    }
	    if { $GidData($STRUCT,VALUE,3) != "-" } {
		set SteelSections::material $GidData($STRUCT,VALUE,3)
	    }

	    GidHelpRecursive $f "Select one steel section and one material here following" \
		    " the EA-95 regulations. Note that the section will be oriented with its" \
		    " y axe pointing to the beam local y' axe"
	    grid $f -row $ROW -column 0 -sticky nsew -columnspan 2 -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	      set STRUCT [lindex $args 1]

	    foreach "name mat" [GiveSelectedValues] break
	    if { ![info exists name] || $name == "" } {
		return [list ERROR "It is necessary to select one steel section"]
	    } else {
		DWLocalSetValue $GDN $STRUCT SteelName $name
		DWLocalSetValue $GDN $STRUCT SteelType $mat
	    }
	    return ""
	}
    }
}

#  proc WriteHeader {} {
#      lappend ::auto_path /tcltk/bwidget-1.3.1
#      package require BWidget
#      pack [frame .f]  -fill both -expand 1
#      set SteelSections::path .
#      SteelSections::Create .f
#      SteelSections::WriteCData [file join $SteelSections::path SteelSections.h]
#      exit
#  }

#  proc WriteTextFile {} {
#      lappend ::auto_path /tcltk/bwidget-1.3.1
#      package require BWidget
#      pack [frame .f]  -fill both -expand 1
#      set SteelSections::path .
#      SteelSections::Create .f
#      SteelSections::WriteTxtData [file join $SteelSections::path SteelSections-new.txt]
#      exit
#  }

