

proc e_ { args } {
    return [eval format $args]
}

namespace eval dimensioner {
    variable fyk
    variable fyd
    variable fcd
    variable steellongs
    variable separations
    variable mcovering
    variable gammac
    variable gammas
}

proc dimensioner::Init { concrete_res } {
    variable fyk
    variable fyd
    variable fcd
    variable steellongs
    variable separations
    variable mcovering

    set separations $ConcretePrefs::separations
    if { $separations == "Automatic" } {
	set separations 0.02
    }
    set mcovering [list given $ConcretePrefs::m1covering]
    if { [lindex $mcovering 1] == "Automatic" } {
	set mcovering [list auto ""]
    }
    set steellongs ""
    foreach i [list 6 8 10 12 14 16 20 25 32 40] {
	if { [set ConcretePrefs::steellong$i] == 1 } {
	    lappend steellongs $i
	}
    }

    switch $ConcretePrefs::steeltype {
	"B400-S" { set fyk 4.0e8 }
	"B500-S" { set fyk 5.0e8 }
	default { error "unknown type" }
    }
    set fck [expr {[string range $concrete_res 3 end]*1e6}]
    set gammac 1.5
    set gammas 1.15

    set fyd [expr {$fyk/double($gammas)}]
    set fcd [expr {$fck/double($gammac)}]

    set comms {
	m22cm2 1e4
	m2cm 1e2
	cm2m 1e-2
	m2mm 1e3
	mm2m 1e-3
	N2kN 1e-3
	Pa2MPa 1e-6
	Nm2kNm 1e-3
    }
    foreach "comm fac" $comms {
	proc $comm val [string map "FAC $fac" { return [expr {$val*FAC}] }]
    }
    
}

proc dimensioner::BeginHTML { title } {
    variable buffer

    set buffer "<html>\n<head>\n<title>$title</title>\n</head>\n<body bgcolor=\"#FFFFFF\">\n"
}
proc dimensioner::EndHTML {} {
    variable buffer

    append buffer "</body>\n</html>\n"

    set file [file join [RamTempDir] dim.html]
    set fout [open $file w]
    puts $fout $buffer
    close $fout
    RamStartFile $file
}

proc dimensioner::WriteHTML { tag buf } {
    variable buffer

    set map {
	 &aacute;
	 &eacute;
	 &iacute;
	 &oacute;
	 &uacute;
	 &ntilde;
	 &middot;
	<h4> <b>
	<H4> <b>
	</h4> </b>
	</H4> </b>
    }
    regsub {[hH]4} $tag b tag
    append buffer "<$tag>[string map $map $buf]</$tag>\n"
}

proc dimensioner::WriteHTMLRed { tag buf } {
    variable buffer

    append buffer "<font color=\"red\">\n"
    WriteHTML $tag $buf
    append buffer "</font>\n"
}

# type can be: support,plate,shell,wall,beam,traction
# if Asmiddle < 0 , it will not be considered (not used by now)
proc dimensioner::CalcMinimumLongitudinal { N b h AsUp AsDown Asmiddle type CanDimension dowrite } {
    variable fyk
    variable fyd
    variable fcd

    set N [expr abs($N)]

    if { $type == "support" } {
	set Amin [expr {4e-3*$b*$h}]
	if { $CanDimension && $AsUp+$Asmiddle+$AsDown < $Amin } {
	    set AsUp [expr $Amin/2.0]
	    set AsDown [expr $Amin/2.0]
	}
	set CSminlong [expr {($AsUp+$Asmiddle+$AsDown)/double($Amin)}]
	if { $dowrite } {
	    WriteHTML h3 [e_ "Cuantas geomtricas mnimas (EHE 42.3.5)"]
	    WriteHTML p [e_ "Comprobacin para pilares."]
	    set buf [e_ "A<sub>s</sub> = %.3gcm<sup>2</sup> &gt; 0.004A<sub>c</sub>=%.3gcm<sup>2</sup> "\
		[m22cm2 [expr {($AsUp+$Asmiddle+$AsDown)}]] [m22cm2 $Amin]]
	    WriteHTML p $buf
	    if { $CSminlong >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	}
    } elseif { $type == "traction" } {
	set Amin [expr {.2*$b*$h*$fcd/double($fyd)}]
	if { $CanDimension && $AsUp+$Asmiddle+$AsDown < $Amin } {
	    set AsUp [expr $Amin/2.0]
	    set AsDown [expr $Amin/2.0]
	}
	set CSminlong [expr {($AsUp+$Asmiddle+$AsDown)/double($Amin)}]
	if { $dowrite } {
	    WriteHTML h3 [e_ "Limitacin traccin (EHE 42.3.4)"]

	    WriteHTML p [e_ "A<sub>s</sub>f<sub>yd</sub> &gt; 0.20*A<sub>c</sub>f<sub>cd</sub>"]
	    WriteHTML p [e_ "%.3g%.3g &gt; 0.20%.3g%.3g" [expr {$AsUp+$Asmiddle+$AsDown}] \
		$fyd [expr {$b*$h}] $fcd]
	    WriteHTML p [e_ "%.3gkN &gt; %.3gkN" [N2kN [expr {($AsUp+$Asmiddle+$AsDown)*$fyd}]] \
		[N2kN [expr {.20*$b*$h*$fcd}]]]
	    if { $CSminlong >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	}
    } else {
	if { $type  == "plate" || $type == "shell" } {
	    if { $fyk < 4.5e8 } { set fac 2.0 } else { set fac 1.8 }
	    set facdown 0.5
	    set facup 0.5
	    set cmpname [e_ "placas"]
	} elseif { $type  == "wall" } {
	    if { $fyk < 4.5e8 } { set fac 1.2 } else { set fac 0.9 }
	    set facdown 1.0
	    set facup 0.3
	    #if(beamtype!=FLEXIONUP_BT && beamtype!=FLEXIONDOWN_BT) facup=1.0;
	     set cmpname [e_ "muros"]
	} elseif { $type  == "beam" } {
	    if { $fyk < 4.5e8 } { set fac 3.3 } else { set fac 2.8 }
	    set facdown 1.0
	    set facup 0.3
	    #if(beamtype!=FLEXIONUP_BT && beamtype!=FLEXIONDOWN_BT) facup=1.0;
	     set cmpname [e_ "vigas"]
	} else { error "unknown type: $type" }

	set Amin [expr {$fac*1e-3*$b*$h}]

	if {$AsDown<$facdown*$Amin && $CanDimension} {
	    set AsDown [expr {$facdown*$Amin}]
	}
	if {$AsUp<$facup*$Amin && $CanDimension } {
	    set AsUp [expr {$facup*$Amin}]
	}
	set CSminlong [expr {$AsDown/double($facdown*$Amin)}]
	set CS [expr {$AsUp/double($facup*$Amin)}]
	if { $CS < $CSminlong && ($CS < 1.0 || $facup>=$facdown) } { set CSminlong $CS }

	if { $dowrite } {
	    WriteHTML h3 [e_ "Cuantas geomtricas mnimas (EHE 42.3.5)"]
	    WriteHTML p [e_ "Comprobacin para %s." $cmpname]

	    set buf [e_ "A<sub>s,inf</sub> = %.3gcm<sup>2</sup> &gt; %.3g%%%.3g/1000A<sub>c</sub>="\
		[m22cm2 $AsDown] [expr {$facdown*100}] $fac]
	    append buf [e_ "%.3gcm<sup>2</sup>" [m22cm2 [expr {$facdown*$Amin}]]]
	    WriteHTML p $buf
	    set buf [e_ "A<sub>s,sup</sub> = %.3gcm<sup>2</sup> &gt; %.3g%%%.3g/1000A<sub>c</sub>="\
		[m22cm2 $AsUp] [expr {$facup*100}] $fac]
	    append buf [e_ "%.3gcm<sup>2</sup>" [m22cm2 [expr {$facup*$Amin}]]]
	    WriteHTML p $buf
	    if { $CSminlong >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	}
    }

    if { $type == "support" || $type == "wall" } {
	set fycd $fyd
	if { $fycd>4.0e8 } { set fycd 4e8 }

	set Amin [expr {.05*$N/double($fycd)}]
	set Amax [expr {.5*$fcd*$b*$h/double($fycd)}]

	if { $AsDown < $Amin && $CanDimension } { set AsDown $Amin }
	if { $AsUp < $Amin && $CanDimension } { set AsUp $Amin }

	set CS0 [expr {$AsDown/double($Amin)}]
	set CS $CS0
	set CS1 [expr {$AsUp/double($Amin)}]
	if { $CS1 < $CS } { set CS $CS1 }
	if { $AsDown > 0 } {
	    set CS2 [expr {$Amax/double($AsDown)}]
	    if { $CS2 < $CS } { set CS $CS2 }
	} else { set CS2 -1 }
	if { $AsUp > 0 } {
	    set CS3 [expr {$Amax/double($AsUp)}]
	    if { $CS3 < $CS } { set CS $CS3 }
	} else { set CS3 -1 }
	if { $CS < $CSminlong } { set CSminlong $CS }

	if { $dowrite } {
	    WriteHTML h3 [e_ "Limitacin compresin (EHE 42.3.3)"]
	    WriteHTML p [e_ "A<sub>s1</sub>f<sub>yc,d</sub> &gt; 0.05N<sub>d</sub>"]
	    WriteHTML p [e_ "%.3g%.3g >= 0.05%.3g" $AsDown $fycd $N]
	    WriteHTML p [e_ "%.3gkN &gt; %.3gkN" [N2kN [expr {$AsDown*$fycd}]] [N2kN [expr {0.05*$N}]]]
	    if { $CS0 >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }

	    WriteHTML p [e_ "A<sub>s2</sub>f<sub>yc,d</sub> &gt; 0.05N<sub>d</sub>"]
	    WriteHTML p [e_ "%.3g%.3g >= 0.05%.3g" $AsUp $fycd $N]
	    WriteHTML p [e_ "%.3gkN &gt; %.3gkN" [N2kN [expr {$AsUp*$fycd}]] [N2kN [expr {0.05*$N}]]]
	    if { $CS1 >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }

	    if { $CS2 > -1 } {
		WriteHTML p [e_ "A<sub>s1</sub>f<sub>yc,d</sub> &lt; 0.5f<sub>cd</sub>A<sub>c</sub>"]
		WriteHTML p [e_ "%.3g%.3g &lt; 0.5%.3g%.3g" $AsDown $fycd $fcd [expr {$b*$h}]]
		WriteHTML p [e_ "%.3gkN &lt; %.3gkN" [N2kN [expr {$AsDown*$fycd}]] \
		    [N2kN [expr {0.5*$fcd*$b*$h}]]]
		if { $CS2 >= 1 } {
		    WriteHTML p [e_ "CUMPLE"]
		} else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	    }
	    if { $CS3 > -1 } {
		WriteHTML p [e_ "A<sub>s2</sub>f<sub>yc,d</sub> &lt; 0.5f<sub>cd</sub>A<sub>c</sub>"]
		WriteHTML p [e_ "%.3g%.3g &lt; 0.5%.3g%.3g" $AsUp $fycd $fcd [expr {$b*$h}]]
		WriteHTML p [e_ "%.3gkN &lt; %.3gkN" [N2kN [expr {$AsUp*$fycd}]] \
		    [N2kN [expr {0.5*$fcd*$b*$h}]]]
		if { $CS3 >= 1 } {
		    WriteHTML p [e_ "CUMPLE"]
		} else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	    }
	}
    } elseif { $type == "beam" } {
	set W [expr {$b*$h/6.0}] ;#=1/12*b*h*h*h/(h/2)
	set Amin [expr {.25*$W/double($h)*$fcd/double($fyd)}]
	
	if { $AsDown < $Amin && $CanDimension } {
	    set AsDown $Amin
	}
	set CS [expr {$AsDown/double($Amin)}]
	if { $CS < $CSminlong } { set CSminlong $CS }
	if { $dowrite } {
	    WriteHTML h3 [e_ "Limitacin flexin (EHE 42.3.2)"]
	    WriteHTML p [e_ "A<sub>s,inf</sub>f<sub>yd</sub> &gt; 0.25W/hf<sub>cd</sub>"]
	    WriteHTML p [e_ "%.3g%.3g &gt; 0.25%.3g/%.3g%.3g" $AsDown [Pa2MPa $fyd] $W $h [Pa2MPa $fcd]]
	    WriteHTML p [e_ "%.3gkN &gt; %.3gkN" [N2kN [expr {$AsDown*$fyd}]] \
		[N2kN [expr {.25*$W/double($h)*$fcd}]]]
	    if { $CS >= 1 } {
		WriteHTML p [e_ "CUMPLE"]
	    } else { WriteHTMLRed p [e_ "NO CUMPLE"] }
	}
    }
    return [list $CSminlong $AsUp $AsDown $Asmiddle]
}

proc dimensioner::CalcBestSteelsPerLength { As } {
    variable steellongs
    variable separations

    set BestD -1
    foreach i $steellongs {
	set Asi [expr {$math::PI*.25*$i*$i*1e-6}] ;# Area of 1 bar
	set sep [expr {$Asi/double($As)}]
	set insep [expr {$sep-$i*1e-3}]
	if { $insep < .02 } { continue }
	if { $insep < $i*1e-3 } { continue }
	if { $sep < $separations } { continue }
	if { $sep > .30 } { set sep .30 }
	
	set sep [format %g [expr {int($sep*100)/100.0}]]
	set sep5 [format %g [expr {int($sep*20)/20.0}]]
	if { $sep >= .15 } { set sep $sep5 }

	# just a weighting factor
	set fac [expr {$Asi/double($sep)/double($As)}]
	if { $sep == $sep5 } { set fac [expr {$fac*.8}] }
	if { $BestD == -1 || $fac < $BestFac } {
	    set BestSep $sep
	    set BestD $i
	    set BestFac $fac
	}
    }
    if { $BestD == -1 } {
	return -1
    } else { return [list $BestD $BestSep] }
}

# it is supposed that most of the steel is needed down (positive moment)
proc dimensioner::SimpleBending { Md b h AsUp AsDown CanDimension dowrite } {
    variable fcd
    variable fyd
    variable mcovering

    set Md [expr {abs($Md)}]

    set dp [lindex $mcovering 1]
    set d [expr {$h-$dp}]
    set U0 [expr {0.85*$fcd*$b*$d}]

    if { $Md <= 0.375*$U0*$d } {
	set Us1 [expr {$U0*(1.0-sqrt(1.0-2.0*$Md/double($U0*$d)))}]
	set Us2 0.0
    } else {
	set Us2 [expr {($Md-0.375*$U0*$d)/double($d-$dp)}]
	set Us1 [expr {0.5*$U0+$Us2}]
    }
    set AsminDown [expr {$Us1/double($fyd)}]
    set AsminUp [expr {$Us2/double($fyd)}]

    if { $AsDown < $AsminDown && $CanDimension } {
	set AsDown $AsminDown
    }
    if { $AsUp < $AsminUp && $CanDimension } {
	set AsUp $AsminUp
    }

    set CS 10 ;# infinite
    set CS1 10
    set CS2 10
    if { $AsminDown > 0.0 } {
	set CS1 [expr {$AsDown/double($AsminDown)}]
	set CS $CS1
    }
    if { $AsminUp > 0.0 } {
	set CS2 [expr {$AsUp/double($AsminUp)}]
	if { $CS2 < $CS && $CS2 < 1.0 } { set CS $CS2 }
    }

    if { $dowrite } {
	WriteHTML h3 [e_ "Agotamiento a flexin, seccin simple rectangular (EHE A8.3.1)"]
	if { $Md == 0 } {
	    set buf [e_ "M<sub>d</sub>=0. No se comprueba"]
	} elseif { $Md <= 0.375*$U0*$d } {
	    set buf [e_ "M<sub>d</sub>=%.3g kNm &lt; 0.375U<sub>0</sub>d=" [Nm2kNm $Md]]
	    append buf [e_ "0.375%.3g%.3g=%.3g kNm" [N2kN $U0] $d [Nm2kNm [expr {0.375*$U0*$d}]]]
	    WriteHTML p $buf

	    set buf [e_ "U<sub>s1</sub>=U<sub>0</sub>(1-&#x221A;(1-2M<sub>d</sub>/(U<sub>0</sub>d)))="]
	    append buf [e_ "%.3g*(1-&#x221A;(1-2%.3g/(%.3g%3g)))=%.3g kN" [N2kN $U0] \
		[Nm2kNm $Md] [N2kN $U0] $d [N2kN $Us1]]
	    WriteHTML p $buf

	    if { $AsminDown <= $AsDown } {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsDown]]
		append buf [e_ "%.3g cm<sup>2</sup> CUMPLE" [m22cm2 $AsminDown]]
		WriteHTML p $buf
	    } else {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsDown]]
		append buf [e_ "%.3g cm<sup>2</sup> NO CUMPLE" [m22cm2 $AsminDown]]
		WriteHTMLRed p $buf
	    }
	} else {
	    set buf [e_ "M<sub>d</sub>=%.3g kNm &gt; 0.375U<sub>0</sub>d=" [Nm2kNm $Md]]
	    append buf [e_ "0.375%.3g%.3g=%.3g" [N2kN $U0] $d [Nm2kNm [expr {0.375*$U0*$d}]]]
	    WriteHTML p $buf

	    set buf [e_ "U<sub>s2</sub>=(M<sub>d</sub>-0.375U<sub>0</sub>d)/(d-d')="]
	    append buf [e_ "(%.3g-0.375%.3g%.3g)/(%.3g-%.3g=%.3g kN" [Nm2kNm $Md] [N2kN $U0] \
		$d $d $dp [N2kN $Us2]]
	    WriteHTML p $buf
	    set buf [e_ "U<sub>s1</sub>=0.5U<sub>0</sub>+U<sub>s2</sub>="]
	    append buf [e_ "0.5%.3g+%.3g=%.3g kN" [N2kN $U0] [N2kN $Us2] [N2kN $Us1]]
	    WriteHTML p $buf

	    if { $AsminDown <= $AsDown } {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsDown]]
		append buf [e_ "%.3g cm<sup>2</sup> CUMPLE" [m22cm2 $AsminDown]]
		WriteHTML p $buf
	    } else {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsDown]]
		append buf [e_ "%.3g cm<sup>2</sup> NO CUMPLE" [m22cm2 $AsminDown]]
		WriteHTMLRed p $buf
	    }
	    if { $AsminUp <= $AsUp } {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsUp]]
		append buf [e_ "%.3g cm<sup>2</sup> CUMPLE" [m22cm2 $AsminUp]]
		WriteHTML p $buf
	    } else {
		set buf [e_ "A<sub>s</sub>=%.3g cm<sup>2</sup> &lt; U<sub>s1</sub>f<sub>yd</sub>=" \
		    [m22cm2 $AsUp]]
		append buf [e_ "%.3g cm<sup>2</sup> NO CUMPLE" [m22cm2 $AsminUp]]
		WriteHTMLRed p $buf
	    }
	}
    }
    return [list $CS $AsUp $AsDown]
}

# it is supposed that most of the steel is needed down (positive moment)
proc dimensioner::SimpleBendingMaxM { b h AsUp AsDown } {
    variable fcd
    variable fyd
    variable mcovering

    set dp [lindex $mcovering 1]
    set d [expr {$h-$dp}]
    set U0 [expr {0.85*$fcd*$b*$d}]
    set Uv [expr {2.0*$U0*$dp/double($d)}]

    set Us1 [expr {$AsDown*$fyd}]
    set Us2 [expr {$AsUp*$fyd}]
    
    if { $Us1-$Us2 < $Uv } {
	set Mu [expr {0.24*$Uv*$dp*($Uv-$Us1+$Us2)*(1.5*$Us1+$Us2)/double((0.6*$Uv+$Us2)*(0.6*$Uv+$Us2))+\
	    $Us1*($d-$dp)}]
    } elseif { $Us1-$Us2 <= 0.5*$U0 } {
	set Mu [expr {($Us1-$Us2)*(1.0-($Us1-$Us2)/double(2.0*$U0))*$d+$Us2*($d-$dp)}]
    } else {
	set alpha [expr {($Us1+0.6*$Us2)/double($U0)}]
	set Mu [expr {4.0/3.0*$Us1*(($alpha+1.2)/double($alpha+sqrt($alpha*$alpha+1.92*\
	    $Us1/double($Us0)))-0.5)*$d+$Us2*($d-$dp)}]
    }
    return $Mu
}

proc dimensioner::SimpleBendingMaxMSection { b h section } {

    foreach "AsUp AsDown" [dimensioner::CalcAreas $section] break

    if { $AsUp > $AsDown } {
	set AsUpL $AsDown
    } else { set AsUpL $AsUp }

    set Mpos [SimpleBendingMaxM $b $h $AsUpL $AsDown]


    if { $AsDown > $AsUp } {
	set AsDownL $AsUp
    } else { set AsDownL $AsDown }

    set Mneg [SimpleBendingMaxM $b $h $AsDownL $AsUp]

    return [list $Mpos $Mneg]
}

# M positive means traction down
proc dimensioner::DimensionForPlateShell { Md b h concrete_res base_steel { dowrite 0 } } {
    variable mcovering
    variable separations

    if { $dowrite } { BeginHTML [e_ "Dimensionamiento placa maciza"] }

    if { $base_steel != "" } {
	foreach "AsUpI AsDownI" [dimensioner::CalcAreas $base_steel] break
	if { $Md < 0 } {
	    set tmp $AsDownI
	    set AsDownI $AsUpI
	    set AsUpI $tmp
	}
    } else {
	set AsUpI 0.0
	set AsDownI 0.0
    }
    Init $concrete_res

    if { [lindex $mcovering 0] == "auto" } {
	set mcovering [list auto $separations]
    }

    foreach "CS AsUp AsDown Asmiddle" [CalcMinimumLongitudinal 0.0 $b $h $AsUpI $AsDownI 0.0 \
	plate 1 $dowrite] break
    if { $CS < 1.0 } { return "" }
    foreach "CS AsUp AsDown" [SimpleBending $Md $b $h $AsUp $AsDown 1 $dowrite] break
    if { $CS < 1.0 } { return "" }

    if { $AsUp > $AsUpI } {
	foreach "bestDUp BestSepUp" [CalcBestSteelsPerLength [expr {$AsUp-$AsUpI}]] break
	if { $bestDUp == -1 } { return "" }
    } else { set bestDUp -1 }
    
    if { $AsDown > $AsDownI } {
	foreach "bestDDown BestSepDown" [CalcBestSteelsPerLength [expr {$AsDown-$AsDownI}]] break
	if { $bestDDown == -1 } { return "" }
    } else { set bestDDown -1 }


    if { [lindex $mcovering 0] == "auto" && $bestDUp != -1 && $bestDDown != -1 } {
	set m [expr {$separations+0.5*$bestDUp*1e-3}]
	set m2 [expr {$separations+0.5*$bestDDown*1e-3}]
	if { $m2 < $m } { set m $m2 }
	set mcovering [list auto $m]
	
	foreach "CS AsUp AsDown" [SimpleBending $Md $b $h $AsUp $AsDown 1 $dowrite] break
	if { $CS < 1.0 } { return "" }
    }

    if { $dowrite } { EndHTML }

    if { $Md < 0 } {
	set tmp $bestDUp
	set bestDUp $bestDDown
	set bestDDown $tmp
	set tmp $BestSepUp
	set BestSepUp $BestSepDown
	set BestSepDown $tmp
    }

    if { $bestDDown != -1 } {
	set blockdown "1D${bestDDown}c/[format %g [expr {$BestSepDown*100}]]cm"
    } else { set blockdown "" }
    if { $bestDUp != -1 } {
	set blockup "1D${bestDUp}c/[format %g [expr {$BestSepUp*100}]]cm"
    } else { set blockup "" }

    return "As=$blockdown A's=$blockup"

#     if { $base_steel == "" } {
#         return "As=$blockdown A's=$blockup"
#     } else {
#         regexp {(As=.*)(A's=.*)} $base_steel {} a b
#         set sec $a
#         if { $blockdown != "" } { append sec ",$blockdown" }
#         append sec " $b"
#         if { $blockup != "" } { append sec ",$blockup" }
#         return $sec
#     }

}

proc dimensioner::CalcAreas { section } {

    set block {1D([0-9]+)c/([0-9.]+)cm}
    for { set i 0 } { $i < 3 } { incr i } { set d($i) "" }
    regexp "As=${block}(,$block)?(,$block)?" $section {} d(0) sep(0) {} d(1) sep(1) {} d(2) sep(2) 
    set AsDown 0
    for { set i 0 } { $i < 3 } { incr i } {
	if { $d($i) != "" } {
	    set AsDown [expr {$AsDown+$math::PI*.25*$d($i)*$d($i)*1e-6*100.0/double($sep($i))}]
	}
    }
    for { set i 0 } { $i < 3 } { incr i } { set d($i) "" }
    regexp "A's=${block}(,$block)?(,$block)?" $section {} d(0) sep(0) d(1) sep(1) d(2) sep(2) 
    set AsUp 0
    for { set i 0 } { $i < 3 } { incr i } {
	if { $d($i) != "" } {
	    set AsUp [expr {$AsUp+$math::PI*.25*$d($i)*$d($i)*1e-6*100.0/double($sep($i))}]
	}
    }
    return [list $AsUp $AsDown]
}
















